using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;

using AXDBLib;

namespace MyRealDWG
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            // we subscribe for this event to get all the log messages
            MyDebugger.OnMessage += new MyDebugger.OnMessageHandler(MyDebugger_OnMessage);

            // fill up the table with the functions and properties so that the user can set if
            // which ones should be overriden
            foreach (DictionaryEntry entry in MyDebugger.mOverrides)
            {
                dgvOverrides.Rows.Add(new object[]{entry.Key, entry.Value});
            }
        }

        #region "Debugging"

        void MyDebugger_OnMessage(
            string msg)
        {
            // we just place the log in the list box
            dgvLog.Rows.Add(new object[]{msg});
        }

        private void dgvOverridesCellValueChanged(
            object sender, 
            DataGridViewCellEventArgs e)
        {
            if (e.RowIndex < 0)
                return;

            // when the user click on the check box of one of the properties or functions,
            // then we update the hashtable which contains the override settings
            DataGridViewCell cellName = dgvOverrides.Rows[e.RowIndex].Cells[0];
            DataGridViewCell cellValue = dgvOverrides.Rows[e.RowIndex].Cells[1];

            MyDebugger.mOverrides[cellName.Value] = cellValue.Value;
        }

        #endregion

        #region ".NET Command"

        /// <summary>
        /// Adds a raster image to the specified database's block table record
        /// </summary>
        /// <param name="func">name of the function to be checked</param>
        public void addPicture(
            Database db, 
            BlockTableRecord ms, 
            Transaction tr)
        {
            // name of the raster image definition
            const string kMyImage = "MY_IMAGE";
            // full path to the image
            string imgPath =
                Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) +
                @"\MyImage.png";

            // check if image exists
            if (!File.Exists(imgPath))
            {
                MessageBox.Show("Image could not be added");
                return;
            }

            // check if the image dictionary exists
            ObjectId dictId = RasterImageDef.GetImageDictionary(db);
            if (dictId.IsNull)
            {
                // image dictionary doesn't exist, create new
                dictId = RasterImageDef.CreateImageDictionary(db);
            }

            // open the image dictionary
            DBDictionary dict = (DBDictionary)tr.GetObject(dictId, OpenMode.ForRead);

            // see if our raster def in the image dict exists, if not, create new
            ObjectId defId;
            if (dict.Contains(kMyImage))
            {
                // get the raster image def
                defId = dict.GetAt(kMyImage);
            }
            else
            {
                // create a raster image definition
                // get the file name of the image
                RasterImageDef def = new RasterImageDef();

                // set its source image
                string myDocs = System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                def.SourceFileName = imgPath;

                // load it
                def.Load();
                dict.UpgradeOpen();
                defId = dict.SetAt(kMyImage, def);

                // let the transaction know
                tr.AddNewlyCreatedDBObject(def, true);
            }

            // now create the raster image that references the definition
            RasterImage img = new RasterImage();
            img.ImageDefId = defId;
            // set its position and extents
            img.Orientation = new CoordinateSystem3d(
                new Point3d(0, -img.ImageSize(false).Y / img.ImageSize(false).X, 0),
                new Vector3d(1, 0, 0),
                new Vector3d(0, img.ImageSize(false).Y / img.ImageSize(false).X, 0));

            // add the image to model space and transaction
            ms.AppendEntity(img);
            tr.AddNewlyCreatedDBObject(img, true);
        }


        /// <summary>
        /// Checks if the user chose to use the overriden version of the given function or property 
        /// </summary>
        /// <param name="func">name of the function to be checked</param>
        public void addText(
            Database db, 
            BlockTableRecord ms, 
            Transaction tr)
        {
            // create a new text entity, and set its properties
            DBText txt = new DBText();
            txt.TextString = "Hello World from .NET !";
            txt.Position = new Point3d(0, 0, 0);

            // add it to the database and transaction
            ms.AppendEntity(txt);
            tr.AddNewlyCreatedDBObject(txt, true);
        }

        public void addTextAndPictureNET()
        {
            string myDocs = System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string dwgPath = myDocs + @"\MyDrawingNET.dwg";
            string str = "";

            if (System.IO.File.Exists(dwgPath))
            {
                // create a database
                // this time we fill it up with default parameters, that's why the first input parameter is true
                // if we were to open a drawing then this parameter should be false
                using (Database db = new Database(false, true))
                {
                    // open drawing
                    db.ReadDwgFile(dwgPath, System.IO.FileShare.ReadWrite, true, "");

                    // set it as the working database
                    HostApplicationServices.WorkingDatabase = db;

                    // let's start a transaction
                    using (Transaction tr = db.TransactionManager.StartTransaction())
                    {
                        // get the block table
                        BlockTable bt = (BlockTable)tr.GetObject(db.BlockTableId, OpenMode.ForRead);

                        // get model space
                        BlockTableRecord ms = (BlockTableRecord)tr.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForRead);

                        foreach (ObjectId id in ms)
                        {
                            Entity ent = (Entity)tr.GetObject(id, OpenMode.ForRead);

                            if (ent is DBText)
                            {
                                DBText text = (DBText)ent;
                                str += text.TextString + "[" + 
                                       text.GeometricExtents.MinPoint.ToString() + ", " +
                                       text.GeometricExtents.MaxPoint.ToString() + "]" + "\n";
                            }
                            else if (ent is RasterImage)
                            {
                                RasterImage img = (RasterImage)ent;
                                RasterImageDef def = (RasterImageDef)tr.GetObject(img.ImageDefId, OpenMode.ForRead);
                                str += img.Path + "[" +
                                       img.GeometricExtents.MinPoint.ToString() + ", " +
                                       img.GeometricExtents.MaxPoint.ToString() + "]" + "\n";
                            }
                        }
                    }

                    // show message
                    MessageBox.Show(str);
                }
            }
            else
            {
                // create a database
                // this time we fill it up with default parameters, that's why the first input parameter is true
                // if we were to open a drawing then this parameter should be false
                using (Database db = new Database(true, true))
                {
                    // set it as the working database
                    HostApplicationServices.WorkingDatabase = db;

                    // let's start a transaction
                    using (Transaction tr = db.TransactionManager.StartTransaction())
                    {
                        // get the block table
                        BlockTable bt = (BlockTable)tr.GetObject(db.BlockTableId, OpenMode.ForRead);

                        // get model space
                        BlockTableRecord ms = (BlockTableRecord)tr.GetObject(bt[BlockTableRecord.ModelSpace], OpenMode.ForWrite);

                        // create a new text entity, and set its properties
                        addText(db, ms, tr);

                        // create a new raster image entity, and set its properties
                        addPicture(db, ms, tr);

                        // commit the changes
                        tr.Commit();
                    }

                    // save the drawing
                    db.SaveAs(dwgPath, DwgVersion.Newest);
                }
            }
        }

        #endregion

        private void btnDotNet_Click(
            object sender, 
            EventArgs e)
        {
            dgvLog.Rows.Clear();

            addTextAndPictureNET();
        }

        #region "COM Command"

        /// <summary>
        /// This creates a new drawing, adds text to it and inserts an image if the given picture already exsists, or if the drawing
        /// already exists, then it just prints out information about the texts and raster images residing in the drawing.
        /// </summary>
        public void addTextAndPictureCOM()
        {
            // it's just a nice way to make sure that we get to the MyDocuments folder, wherever it is
            string myDocs = System.Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            string imgPath = myDocs + @"\MyImage.png";
            string dwgPath = myDocs + @"\MyDrawingCOM.dwg";
            string str = "";

            // check if the image exists
            if (System.IO.File.Exists(dwgPath))
            {
                // create a new empty database
                using (Database db = new Database(false, true))
                {
                    // load drawing
                    db.ReadDwgFile(dwgPath, System.IO.FileShare.ReadWrite, true, "");

                    // set the working database
                    HostApplicationServices.WorkingDatabase = db;

                    // <ActiveX> 
                    AcadDatabase acadDb = (AcadDatabase)db.AcadDatabase;

                    foreach (AcadEntity ent in acadDb.ModelSpace)
                    {
                        if (ent is AcadText)
                        {
                            AcadText text = (AcadText)ent;

                            object minPt = new double[3], maxPt = new double[3];
                            text.GetBoundingBox(out minPt, out maxPt);
                            str += text.TextString + "[" + (new Point3d((double[])minPt)).ToString() + "]" + "\n";
                        }
                        else if (ent is AcadRasterImage)
                        {
                            AcadRasterImage img = (AcadRasterImage)ent;

                            object minPt = new double[3], maxPt = new double[3];
                            img.GetBoundingBox(out minPt, out maxPt);
                            str += img.ImageFile + "[" + (new Point3d((double[])minPt)).ToString() + "]" + "\n";
                        }
                    }

                    // </ActiveX> 

                    MessageBox.Show(str);
                }
            }
            else
            {
                using (Database db = new Database(true, true))
                {
                    // set it as the working database
                    HostApplicationServices.WorkingDatabase = db;

                    // <ActiveX> 
                    AcadDatabase acadDb = (AcadDatabase)db.AcadDatabase;

                    double[] pt = new double[3] { 0, 0, 0 };
                    acadDb.ModelSpace.AddText("Hello World from ActiveX !", pt, .2);

                    if (File.Exists(imgPath))
                    {
                        AcadRasterImage img = acadDb.ModelSpace.AddRaster(myDocs + @"\MyImage.png", pt, 1, 0);
                        pt[1] = -img.ImageHeight;
                        img.Origin = pt;
                    }
                    else
                        MessageBox.Show("Image could not be added");
                    // </ActiveX> 

                    // save the drawing
                    db.SaveAs(dwgPath, DwgVersion.Newest);
                }
            }
        }

        #endregion

        private void btnCom_Click(
            object sender,
            EventArgs e)
        {
            dgvLog.Rows.Clear();

            addTextAndPictureCOM(); 
        }
    }
}